# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""
Dict of schemas version used by application.
"""

SCHEMAS = {
    "ServiceRoot": "ServiceRoot.v1_3_1.json",
    "ChassisCollection": "ChassisCollection.json",
    "Chassis": "Chassis.v1_7_0.json",
    "CollectionCapabilities": "CollectionCapabilities.v1_0_0.json",
    "CompositionService": "CompositionService.v1_0_1.json",
    "ComputerSystemCollection": "ComputerSystemCollection.json",
    "ComputerSystem": "ComputerSystem.v1_5_0.json",
    "Drive": "Drive.v1_4_0.json",
    "ManagerCollection": "ManagerCollection.json",
    "Manager": "Manager.v1_4_0.json",
    "EthernetInterface": "EthernetInterface.v1_4_0.json",
    "EthernetInterfaceCollection": "EthernetInterfaceCollection.json",
    "EventService": "EventService.v1_1_0.json",
    "EventDestination": "EventDestination.v1_3_0.json",
    "Event": "Event.v1_2_1.json",
    "Thermal": "Thermal.v1_4_0.json",
    "StorageCollection": "StorageCollection.json",
    "Storage": "Storage.v1_4_0.json",
    "Message": "Message.v1_0_5.json",
    "NetworkInterfaceCollection": "NetworkInterfaceCollection.json",
    "NetworkPortCollection": "NetworkPortCollection.json",
    "NetworkDeviceFunctionCollection": "NetworkDeviceFunctionCollection.json",
    "NetworkDeviceFunction": "NetworkDeviceFunction.v1_2_1.json",
    "NetworkInterface": "NetworkInterface.v1_1_0.json",
    "NetworkAdapterCollection": "NetworkAdapterCollection.json",
    "NetworkAdapter": "NetworkAdapter.v1_1_0.json",
    "NetworkPort": "NetworkPort.v1_1_0.json",
    "Processor": "Processor.v1_3_0.json",
    "ProcessorCollection": "ProcessorCollection.json",
    "ResourceBlockCollection": "ResourceBlockCollection.json",
    "ResourceBlock": "ResourceBlock.v1_1_0.json",
    "VLanNetworkInterface": "VLanNetworkInterface.v1_1_1.json",
    "VLanNetworkInterfaceCollection": "VLanNetworkInterfaceCollection.json",
    "ZoneCollection": "ZoneCollection.json",
    "Zone": "Zone.v1_2_0.json",
    "Session": "Session.v1_1_0.json",
    "SessionCollection": "SessionCollection.json",
    "EventDestinationCollection": "EventDestinationCollection.json",
    "SessionService": "SessionService.v1_1_3.json",
    "VolumeCollection": "VolumeCollection.json",
    "Volume": "Volume.v1_0_3.json"
}

REGISTRY = {
    "Base": "Base.1.1.0.json"
}
