# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__title__ = 'oneview-redfish-toolkit'

__version__ = '0.4.0'

__copyright__ = '(C) Copyright (2018-2019) Hewlett Packard Enterprise '\
    'Development LP'

__license__ = 'Apache License 2.0'


def get_copyright():
    return __copyright__


def version():
    return __version__
