@echo This script assumes that the python executables needed by this system is reacheble via PATH environment variable
@echo It also assumes that all required libs are installed and if necessary the virtualenv to run the server is activated

python oneview_redfish_toolkit/app.py
